package kaka.cakelight;

import kaka.cakelight.mode.Mode;
import org.jnativehook.GlobalScreen;
import org.jnativehook.NativeHookException;
import org.jnativehook.keyboard.NativeKeyAdapter;
import org.jnativehook.keyboard.NativeKeyEvent;
import org.jnativehook.mouse.NativeMouseEvent;
import org.jnativehook.mouse.NativeMouseMotionAdapter;

import java.util.Objects;
import java.util.Stack;

public class CakeLight {
    private Configuration config;
    private Stack<Mode> modes = new Stack<>();
    private LedController ledController;

    public CakeLight(Configuration config, LedController ledController) {
        this.config = config;
        this.ledController = ledController;
        Color.calculateGammaCorrection(config.gamma);
    }

    public void setMode(Mode mode) {
        cleanup();
        pushMode(mode);
    }

    public void cleanup() {
        while (popMode() != null);
    }

    public void pushMode(Mode mode) {
        Objects.requireNonNull(mode);
        if (!modes.isEmpty()) {
            pauseMode(modes.peek());
        }
        modes.push(mode);
        startMode(mode);
        // TODO: create a composite fading mode of top of stack and new mode
    }

    public Mode popMode() {
        if (modes.size() > 1) {
            Mode mode = modes.pop();
            stopMode(mode);
            resumeMode(modes.peek());
            return mode;
        }
        return null;
        // TODO: create a composite fading mode of popped mode and top of stack, unless doing cleanup
    }

    private void startMode(Mode mode) {
        mode.setFrameListener(ledController::onFrame);
        mode.enter(config);
    }

    private void pauseMode(Mode mode) {
        mode.pause();
    }

    private void resumeMode(Mode mode) {
        mode.resume();
    }

    private void stopMode(Mode mode) {
        mode.setFrameListener(ledFrame -> {}); // To avoid any frame being sent to the controller while the thread is exiting
        mode.exit();
    }

    public void startLoop() {
        Console console = Console.start(this, config);
        PipeController.start(console);
//        initNativeHook();
        // TODO
//        FrameGrabber grabber = FrameGrabber.from(config);
//        grabber.prepare();
//        VideoFrame frame = grabber.grabFrame();
//        double time = 0;
//        for (int i = 0; i < 100; i++) {
//            time += timeIt("frame", () -> grabber.grabFrame());
//        }
//        System.out.println("time = " + time);
//        grabber.close();
//	byte[] data = frame.getData();
//	saveFile(data, "/home/kaka/test.img");
    }

    private void initNativeHook() {
        try {
            GlobalScreen.registerNativeHook();
            GlobalScreen.addNativeKeyListener(new NativeKeyAdapter() {
                @Override
                public void nativeKeyPressed(NativeKeyEvent e) {
                    System.out.println("key code = " + e.getKeyCode() + ", key text = '" + NativeKeyEvent.getKeyText(e.getKeyCode()) + "'");
                }
            });
            GlobalScreen.addNativeMouseMotionListener(new NativeMouseMotionAdapter() {
                @Override
                public void nativeMouseMoved(NativeMouseEvent e) {
                    System.out.println("mouse point = " + e.getPoint());
                }
            });
        } catch (NativeHookException e) {
            e.printStackTrace();
        }
    }

    public void turnOff() {
        cleanup();
        ledController.onFrame(LedFrame.from(config).fillColor(0, 0, 0));
    }
}
