package kaka.cakelight.mode;

import kaka.cakelight.Color;
import kaka.cakelight.LedFrame;

public class SunriseMode extends AmbientMode {
    private int durationSeconds;

    public SunriseMode(int durationSeconds) {
        this.durationSeconds = durationSeconds;
    }

    @Override
    protected void updateFrame(LedFrame frame, long time, int count) {
        double progress = clamp(time / (durationSeconds * 1000.0));
        double elevation = 2.0 - progress;
        double radius = progress * 2.12;
        for (int i = 0; i < config.leds.getCount(); i++) {
            double x = frame.xOf(i);
            double y = frame.yOf(i);
            double distance = distanceFromSun(x, y, elevation);
            double r = clamp(1.0 - distance + radius);
            double g = clamp(0.5 - distance + radius);
            double b = clamp(0.0 - distance + radius);
            frame.setLedColor(i, Color.rgb(r, g, b));
        }
    }

    private double clamp(double value) {
        return Math.min(Math.max(value, 0), 1.0);
    }

    private double distanceFromSun(double x, double y, double elevation) {
        return Math.sqrt(Math.pow(0.5 - x, 2) + Math.pow(elevation - y, 2));
    }
}
